<?php

namespace Drupal\media_imagick\Plugin\media\SourceDecorator;

use Drupal\media\MediaInterface;
use Drupal\media_imagick\MediaSourceDecoratorBase;

/**
 * Class Imagick image media source decorator.
 */
class Imagick extends MediaSourceDecoratorBase {

  /**
   * The decorated Image source plugin.
   *
   * @var \Drupal\media\Plugin\media\Source\Image
   */
  protected $decorated;

  /**
   * A static cache for Imagick objects.
   *
   * @var \Imagick[]
   */
  protected $imagickCache;

  /**
   * Get Imagick object for a media item.
   *
   * @param \Drupal\media\MediaInterface $media
   *   The media object to get Imagick for. As this is expensive, it will be
   *   statically cached.
   *
   * @return \Imagick
   *   The imagick object.
   */
  protected function getImagick(MediaInterface $media) {
    $source_field = $this->configuration['source_field'];
    /** @var \Drupal\file\FileInterface $file */
    $file = $media->{$source_field}->entity;
    $uri = $file->getFileUri();
    $mtime = filemtime($uri);

    $imagick =& $this->imagickCache["$uri:$mtime"];
    if (empty($imagick)) {
      $imagick = new \Imagick($uri);
    }
    return $imagick;
  }

  /**
   * {@inheritdoc}
   */
  public function getMetadataAttributes() {
    $attributes = $this->decorated->getMetadataAttributes();

    $attributes += [
      'media_imagick_format' => $this->t('Format (JPEG, PNG, etc.)'),
      'media_imagick_colorspace' => $this->t('Colorspace (RGB, CMYk, etc.)'),
      'media_imagick_icc' => $this->t('ICC color model'),
      'media_imagick_colorspace_icc' => $this->t('Colorspace/ICC color model'),
      'media_imagick_resolution' => $this->t('Resolution in DPI'),
      // Overridden from Image.
      'mime' => $this->t('Mimetype, from Imagick'),
      ($this->decorated)::METADATA_ATTRIBUTE_WIDTH => $this->t('Width, from Imagick'),
      ($this->decorated)::METADATA_ATTRIBUTE_HEIGHT => $this->t('Height, from Imagick'),
    ];

    return $attributes;
  }

  /**
   * {@inheritdoc}
   */
  public function getMetadata(MediaInterface $media, $name) {
    // Get the file and image data.
    /** @var \Drupal\file\FileInterface $file */
    $file = $media->get($this->configuration['source_field'])->entity;
    // If the source field is not required, it may be empty.
    if ($file && class_exists('\Imagick')) {
      switch ($name) {
        case 'media_imagick_format':
          return $this->getImagick($media)->getImageFormat();

        case 'media_imagick_colorspace':
          // Do it like this, as getImageColorspace() returns constants.
          $info = $this->getImagick($media)->identifyImage();
          $colorspace = isset($info['colorSpace']) ? $info['colorSpace'] : '?';
          return $colorspace;

        case 'media_imagick_icc':
          $properties = $this->getImagick($media)->getImageProperties();
          $icc = isset($properties['icc:model']) ? $properties['icc:model'] : '?';
          return $icc;

        case 'media_imagick_colorspace_icc':
          $info = $this->getImagick($media)->identifyImage();
          $colorspace = isset($info['colorSpace']) ? $info['colorSpace'] : '?';
          $properties = $this->getImagick($media)->getImageProperties();
          $icc = isset($properties['icc:model']) ? $properties['icc:model'] : '?';
          $colorspace_icc = "$colorspace/$icc";
          return $colorspace_icc;

        case 'media_imagick_resolution':
          $resolution = $this->getImagick($media)->getImageResolution();
          return (
            isset($resolution['x'])
            && isset($resolution['y'])
            && $resolution['x'] === $resolution['y']
          ) ? $resolution['x'] : NULL;

        case 'mime':
          return $this->getImagick($media)->getImageMimeType();

        case ($this->decorated)::METADATA_ATTRIBUTE_WIDTH:
          $width = $this->getImagick($media)->getImageWidth();
          return $width ? $width : FALSE;

        case ($this->decorated)::METADATA_ATTRIBUTE_HEIGHT:
          $height = $this->getImagick($media)->getImageHeight();
          return $height ? $height : FALSE;

      }
    }

    return parent::getMetadata($media, $name);
  }

}
