<?php

namespace Drupal\media_imagick;

use Drupal\Core\Plugin\DefaultPluginManager;
use Drupal\Core\Plugin\Factory\ContainerFactory;

/**
 * Plugin decorator helper.
 *
 * @todo Make this a PluginDecoratorPlugin.
 * @todo Upstream idea of a PluginDecoratorPluginDeriverDecorator
 *
 * @see \Drupal\media_imagick\PluginDecoratorInitTrait
 */
class PluginDecoratorHelper {

  /**
   * Alter a plugin definition.
   *
   * In your decorating class, use this:
   * @code
   *   $this->decorated = PluginDecorator::getDecorated($configuration, $id, $definition);
   * /@code
   *
   * Note that this uses some very unholy hacks to create an instance of the
   * decorated plugin aside from the plugin manager. Thus it will only work for
   * DefaultPluginManager descendants that use ContainerFactory or
   * DefaultFactory.
   *
   * @param array $definitions
   *   The definitions to alter.
   * @param string $id
   *   The ID to alter.
   * @param string $managerId
   *   The ID of the plugin manager.
   * @param string $class
   *   The decorating class.
   * @param string $provider
   *   The providing module name.
   */
  public static function alterDefinition(array &$definitions, $id, $managerId, $class, $provider) {
    if (!isset($definitions[$id])) {
      return;
    }
    $innerDefinition = $definitions[$id];
    $definitions[$id] = [
      'id' => $id,
      'class' => $class,
      'provider' => $provider,
      'decorator_info' => [
        'manager' => $managerId,
        'definition' => $innerDefinition,
      ],
    ] + $innerDefinition;
  }

  /**
   * Get decorated plugin.
   *
   * @param array $configuration
   *   The Plugin configuration.
   * @param string $id
   *   The plugin id.
   * @param mixed $definition
   *   The Plugin definition.
   *
   * @return mixed
   *   The decorated plugin.
   */
  public static function getDecorated(array $configuration, $id, $definition) {
    $managerId = $definition['decorator_info']['manager'];
    $innerDefinition = $definition['decorator_info']['definition'];
    /** @var \Drupal\Core\Plugin\DefaultPluginManager $manager */
    $manager = \Drupal::service($managerId);
    return static::createInstance($manager, $id, $innerDefinition, $configuration);
  }

  /**
   * Helper to create an instance providing plugin definition.
   *
   * This is a forked copy of ContainerFactory that allows the plugin definition
   * to be passed as parameter. As it's private to the factory, we can't check
   * the interface here.
   *
   * @todo Upstream what is needed to end this unholy hack!
   *
   * @param \Drupal\Core\Plugin\DefaultPluginManager $plugin_manager
   *   The plugin manager. Must be default factory as ::getPluginClass does not
   *   have an interface.
   * @param string $plugin_id
   *   The plugin id.
   * @param mixed $plugin_definition
   *   The plugin definition. This is different from the original method.
   * @param array $configuration
   *   The plugin configuration.
   *
   * @return mixed
   *   An instance of the plugin.
   *
   * @see \Drupal\Core\Plugin\Factory\ContainerFactory::createInstance
   */
  protected static function createInstance(DefaultPluginManager $plugin_manager, $plugin_id, $plugin_definition, array $configuration = []) {
    // @see \Drupal\Core\Plugin\DefaultPluginManager::getFactory
    $factory = new ContainerFactory($plugin_manager);

    $plugin_class = $factory::getPluginClass($plugin_id, $plugin_definition);

    // If the plugin provides a factory method, pass the container to it.
    if (is_subclass_of($plugin_class, 'Drupal\Core\Plugin\ContainerFactoryPluginInterface')) {
      return $plugin_class::create(\Drupal::getContainer(), $configuration, $plugin_id, $plugin_definition);
    }

    // Otherwise, create the plugin directly.
    return new $plugin_class($configuration, $plugin_id, $plugin_definition);
  }

}
